import os
import sys
import click
import shutil
import json
from summer import app

if __name__ == '__main__':
    sys.path.insert(0, os.path.abspath(os.curdir))


@click.group()
def cli():
    """
    Summer Server 启动命令行管理工具
    """


# @click.command()
# def create_tables():
#     """
#     删除当前数据库内容，并创建数据库
#     :param:
#         * 无
#     :return:
#         * 无
#     """
#     click.echo(click.style('Create db tables.', fg='red'))
#     db.drop_all()
#     db.create_all()
#     click.echo(click.style('Tables created success.', fg='blue'))


@cli.command('start')
@click.option('--reload', default=False, type=click.BOOL, help='是否自动重新加载服务')
@click.option('--ip', default='127.0.0.1', type=click.STRING, help='IP地址', prompt='ip地址', required=False)
@click.option('--port', default='8080', type=click.STRING, help='端口号', prompt='端口号', required=False)
def start_server(reload):
    """
    启动 Server
    :param:
        * 无
    :return:
        * 无
    """
    click.echo(click.style('server start', fg='blue'))
    app.run(use_reloader=reload, host='127.0.0.1', port=app.config['IP_PORT'])


@cli.command('create')
@click.option('--name', '-n', default='py_summer_demo', type=click.STRING, help='项目名称', prompt='项目名称',
              required=True)
@click.option('--path', '-d', default='./', type=click.STRING, help='项目路径', prompt='项目路径', required=True)
def create(name, path):
    """
    创建新项目
    :param:
        * name(str): 项目路径
        * path(str): 项目名称
    :return:
        无
    """
    project_full_path = os.path.join(path, name)
    if os.path.exists(project_full_path):
        shutil.rmtree(project_full_path)
    os.makedirs(project_full_path)

    with open(os.path.join(os.path.dirname(__file__), 'tpl', 'config.json'), "r") as f:
        config = json.load(f)
    structure = config.get('structure')
    _load_config(structure, project_full_path)


def _load_config(structure, project_full_path):
    """
    加载生成代码的配置
    :param:
        * structure(str): 配置文件
        * project_full_path(str): 项目路径
    :return:
        无
    """
    for item in structure:
        name = item.get('name')
        is_folder = item.get('is_folder')
        child = item.get('child')
        current_path = os.path.join(project_full_path, name)

        if is_folder:
            _create_folder(current_path)
            _load_config(child, current_path)
        else:
            tpl = item.get('content')
            _create_file(current_path, tpl)


def _create_folder(folder_path):
    """
    创建文件夹
    :param:
        * folder_path(str): 文件夹地址
    :return:
        * 无
    """
    if os.path.exists(folder_path):
        shutil.rmtree(folder_path)
    else:
        os.makedirs(folder_path)


def _create_file(file_path, tpl_name):
    """
    根据模版，生成文件
    :param:
        * file_path(str): 文件地址
        * tpl_name(str): 模版名称
    :return:
        * 无
    """
    tpl_file_path = os.path.join(os.path.dirname(__file__), 'tpl', tpl_name)
    _buffer = open(tpl_file_path, 'rb')
    with open(file_path, 'wb') as f:
        f.write(_buffer.read())
    _buffer.close()


# server 主要执行过程
if __name__ == '__main__':
    cli.add_command(start_server)
    # cli.add_command(create_tables)
    cli.add_command(create)
    cli()




from fishbase import fish_file as fff
import os


class Config(object):
    """
    配置文件类
    """
    DEBUG = False
    TESTING = False
    CURRENT_PATH = os.path.dirname('.')


def get_db_sqlite_uri(db_name, db_path=Config.CURRENT_PATH):
    """
    获取 sqlite 地址

    :param:
        * db_name(str): 数据库名称
    :return:
        * db_sqlite_uri(str): sqlite 链接地址
    """
    db_sqlite_uri = 'sqlite:///' + fff.get_abs_filename_with_sub_path(db_path, db_name)[1]
    return db_sqlite_uri


class DevelopmentConfig(Config):
    """
    开发环境配置类
    """
    TESTING = False
    DEBUG = True
    SQLITE_NAME = os.environ.get('SQLITE_NAME', default='dev.sqlite')
    SQLALCHEMY_DATABASE_URI = get_db_sqlite_uri(SQLITE_NAME)
    PORT = 8080
    IP = '0.0.0.0'
    SQLALCHEMY_TRACK_MODIFICATIONS = True
    SQLALCHEMY_COMMIT_TEARDOWN = True


class TestingConfig(Config):
    """
    测试环境配置类
    """
    TESTING = True
    PORT = 8080
    IP = '0.0.0.0'



from summer import Base, db
from sqlalchemy import Column, Integer, String


class UserModel(Base):
    """
    建立用户信息表
    """
    __tablename__ = 'test_table'
    uid = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(64), unique=True, nullable=False, index=True)
    score = Column(Integer)

    @staticmethod
    def get_info_by_id(uid):
        """
        根据uid获取信息

        :param:
          * uid(str): 用户 id
        :return:
          * 查询结果
        """
        return db.query(UserModel).filter_by(uid=uid).first()



import os
from flask import Flask, Blueprint
from summer.config import DevelopmentConfig, TestingConfig
from sqlalchemy.ext.declarative import declarative_base


class Summer(Flask):
    """
    申明一个 Summer 类，目前继承自 Flask
    """

    @staticmethod
    def load_conf(application):
        """
        加载自定义配置到当前的 application 对象中

        :param:
            * application(object): application 对象
        :return:
           无
        """
        server_env = os.environ.get('SERVER_ENV', default='development')
        if server_env == 'development':
            application.config.from_object(DevelopmentConfig)
        elif server_env == 'testing':
            application.config.from_object(TestingConfig)

    def create_server(self, test_config=None):
        """
        根据 application 对象创建 server

        :param:
            * test_config(dict): application 对象
        :return:
           * application(object) application 对象
        """

        application = Flask(__name__, instance_relative_config=True)
        application.config.from_mapping(
            SECRET_KEY='secret_key_only_test'
        )

        if test_config is None:
            # 尝试从 config.py 中读入配置文件
            self.load_conf(application)
        else:
            # 如果传入 test 配置，则使用
            application.config.update(test_config)
        return application


# Summer 提供默认的app对象、db对象、同时维护套默认配置对象
app = Summer(__name__, instance_relative_config=True).create_server()

Base = declarative_base()



import pytest
from summer import app as testapp, db, engine, Base
import os
import tempfile


@pytest.fixture
def app():
    @testapp.route('/hello')
    def get_user():
        return 'py_summer'

    db_fd, db_path = tempfile.mkstemp()
    yield testapp, db_path
    os.close(db_fd)
    os.unlink(db_path)


@pytest.yield_fixture()
def db():
    Base.metadata.create_all(engine)
    yield db
    Base.metadata.drop_all(engine)


@pytest.yield_fixture
def client(app):
    with app[0].test_client() as client:
        yield client



from setuptools import setup
import io
import re

with io.open('README.rst', 'rt', encoding='utf8') as f:
    readme = f.read()

with io.open('summer/__init__.py', 'rt', encoding='utf8') as f:
    version = re.search(r'__version__ = \'(.*?)\'', f.read()).group(1)

setup(
    name='py_summer',
    version=version,
    install_requires=['pytest', 'Flask', 'SQLAlchemy', 'fishbase', 'click'],
    url='https://github.com/chinapnr/py-summer',
    license='MIT',
    author='David Yi',
    author_email='wingfish@gmail.com',
    description='一个快速生成 Python Web 项目框架的工具',
    long_description=readme,
    packages=['summer', 'demo', 'test'],
    package_data={
        "summer": ["tpl/*", "tpl/.*"],
    },
    entry_points={"console_scripts": ["summer=summer:cli"]},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ]

)



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from summer.app import app, Base, Summer, Blueprint
from summer.command import cli

engine = create_engine(app.config.get('SQLALCHEMY_DATABASE_URI'))
Session = sessionmaker(bind=engine)
db = Session()

__version__ = '0.0.1'
